--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

	------------------------------------------------------------------------------------------------------------
	--				  Init. Game Assets					  	  --
	------------------------------------------------------------------------------------------------------------

local AddonName, NS = ...
local L = NS.TEXTS

local floor = math.floor
local len, upper, sub = string.len, string.upper, string.sub

----------------------------------------------------------------------------------------------------------
--						Colours							--
----------------------------------------------------------------------------------------------------------
------------------------------------
--         Colour Setup           --
------------------------------------
local CombatColors = {
	Combat = { r=0.717, g=0, b=0.047, a=1,},
	NotCombat = { r=1, g=1, b=1, a=1, },
	CombatDefault = { r=0.717, g=0, b=0.047, a=1,},
	NotCombatDefault = { r=1, g=1, b=1, a=1, },
}
NS:AddOption("CombatColors", CombatColors)

local CombatColorEvents = {
	PLAYER_ENTER_COMBAT = "Combat",
	PLAYER_REGEN_DISABLED = "Combat",
	PLAYER_ENTERING_WORLD = "NotCombat",
	PLAYER_LEAVE_COMBAT = "NotCombat",
	PLAYER_REGEN_ENABLED = "NotCombat",
}

local function LeftCombat(self, event)
	if not self.NoRegen or (self.NoRegen and event == "PLAYER_REGEN_ENABLED") then
		return true
	end
end
local ColourFrames = {}
local MetersDisplay
local function CombatColorsApply(self, event, ...)
	local Combat = CombatColorEvents[event]
--	if Combat == "NotCombat" and InCombatLockdown() then
--		return
--	end
	local Color = CombatColors[Combat]
	if event == "PLAYER_REGEN_DISABLED" then
		self.NoRegen = true
	end
	if Combat == "Combat" or LeftCombat(self, event) then --not self.NoRegen or (self.NoRegen and event == "PLAYER_REGEN_ENABLED") then
		for i=1, #ColourFrames do
			ColourFrames[i]:SetColor(Color.r, Color.g, Color.b)
		end
	end
	if MetersDisplay == 2 then -- 2=incombat
		if Combat == "Combat" then
			NS:GetFrame("HotSpots", 8):ToggleMeterFrame("show")
		else
			if LeftCombat(self, event) then --not self.NoRegen or (self.NoRegen and event == "PLAYER_REGEN_ENABLED") then
				NS:GetFrame("HotSpots", 8):ToggleMeterFrame("hide")
			end
		end
	end
	if self.NoRegen and event == "PLAYER_REGEN_ENABLED" then
		self.NoRegen = nil
	end
--	NS:GetFrame("Context", "Parent").Texture:SetVertexColor(Color.r, Color.g, Color.b)
end
NS:AddOption("CombatColorsApply", CombatColorsApply)

local function CombatColorsInit()
	MetersDisplay = NS:GetSettings("Addons").Meters.inactive
	if not MetersDisplay then
		MetersDisplay = NS:GetSettings("Addons").Meters.show
	else
		MetersDisplay = 0
	end
	local settings = NS:GetSettings("UI").skin
--	if settings.colorcombat then
		CombatColors.Combat.r, CombatColors.Combat.g, CombatColors.Combat.b, CombatColors.Combat.a = settings.colorcombat.r, settings.colorcombat.g, settings.colorcombat.b, settings.colorcombat.a
--	end
--	if settings.colornotcombat then
		CombatColors.NotCombat.r, CombatColors.NotCombat.g, CombatColors.NotCombat.b, CombatColors.NotCombat.a = settings.colornotcombat.r, settings.colornotcombat.g, settings.colornotcombat.b, settings.colornotcombat.a
--	end
	for k,_ in pairs(CombatColorEvents) do
		if settings.color then
			NS:RegisterEvent(k, NS:GetFrame("Base", 1), CombatColorsApply)
		else
			NS:UnregisterEvent(k, NS:GetFrame("Base", 1), CombatColorsApply)
		end
	end
end
NS:AddOption("CombatColorsInit", CombatColorsInit)

----------------------------------------------------------------------------------------------------------
--					Delete Check							--
----------------------------------------------------------------------------------------------------------
local DELETE_GOOD_ITEM
local function ConfirmDelete()
	if not DELETE_GOOD_ITEM then
		DELETE_GOOD_ITEM = StaticPopupDialogs.DELETE_GOOD_ITEM
	end
	--- Get rid of the "type in DELETE to destroy good items ---
	--- Set the destroy dialog to also accept Enter Key as well as mouse click on OK ---
	if NS:GetSettings("UI").confirmdelete then
		StaticPopupDialogs.DELETE_GOOD_ITEM = DELETE_GOOD_ITEM
		StaticPopupDialogs.DELETE_GOOD_ITEM.enterClicksFirstButton = nil
	else
		StaticPopupDialogs.DELETE_GOOD_ITEM=StaticPopupDialogs.DELETE_ITEM
		StaticPopupDialogs.DELETE_GOOD_ITEM.enterClicksFirstButton = 1
	end
end
NS:AddOption("ConfirmDelete", ConfirmDelete)

----------------------------------------------------------------------------------------------------------
--						Skin							--
----------------------------------------------------------------------------------------------------------
local SkinFrames = {
	LEFT = 1,
	CENTERLEFT = 2,
	CENTERRIGHT = 3,
	RIGHT = 4,
	CAVE = "CAVE",
	MINIMAP = "MINIMAP",
}

local function LoadSkin(configmode)
	local Daily, Logon = 1, 2
	local Settings = NS:GetSettings("UI").skin
	if not configmode and Settings.random then
		local Today = date("*t")
		Today = Today.year + Today.yday
		if Settings.random then
			if not Settings.daily or (not Settings.lastdate or Settings.lastdate < Today) then
				local SkinList = NS.GetAccessoryList("Skin")				
				local Skins = {}
				for k, v in pairs(SkinList) do
					if not Settings.ignore[k] then
						tinsert(Skins, k)
					end
				end
				Settings.skin = Skins[random(1, #Skins)]
				if Settings.daily then
					Settings.lastdate = Today
				end
			end
		end
	end
	local Skin = Settings.skin or NS.GetDefaultAccessory("Skin")
	local SkinList = NS.GetAccessorySettings(Skin, "Skin")
	if not SkinList then
		return
	end
	NS:GetFrame("Base", "Cave"):SetFrameLevel(2)
	local VPOffset = NS:GetOption("Metrics").RampartTop --2.1
	local CaveY = 1.7
	local f = NS:GetFrame("Base", 1) 
	local b = NS:GetFrame("HotSpots", 4)
--	if VPOffset == 0 then
--		f.VP = 0
--	else
--		f.VP = b:GetBottom()+(b:GetHeight()/VPOffset)
--	end
	f.VP = VPOffset
	local Path = "Interface\\AddOns\\"..Skin .."\\" 
	for k, v in pairs(SkinList) do
		local SkinID
		if SkinFrames[upper(k)] then
			SkinID = SkinFrames[upper(k)]
			if type(v) ~= "table" then
				local t
				if type(v) ~= "boolean" and SkinFrames[upper(k)] then
					t = Path..v
				end
				local Frame = NS:GetFrame("Base", SkinID)
				if Frame then
					Frame:SetTexture(t)
--					FrameList[SkinID]:SetTexture(t)
--				else
--					FrameList[upper(k)]:SetTexture(t)
				end
				if SkinID == 4 then
					NS:GetFrame("Context", "Parent").Texture:SetTexture(t) -- Skin Context background
				end
			end
		end
	end
	local HotSpots = NS:GetFrame("HotSpots")
	for k, v in pairs(HotSpots) do
		local Text = v.Text
		local Font, Size = Text:GetFont()
		if not SkinList.Text or not SkinList.Text.color then
			Text:SetTextColor(1, 1, 1)
		end
		if not SkinList.Text or not SkinList.Text.flags then
			Text:SetFont(Font, Size, "")
		end
		if SkinList.Text and SkinList.Text.color then
			Text:SetTextColor(SkinList.Text.color.r or 0, SkinList.Text.color.g or 0, SkinList.Text.color.b or 0)
		end
		if SkinList.Text and SkinList.Text.flags then
			local Flags = ""
			local Count = #SkinList.Text.flags
			for i=1, Count do
				Flags = Flags .. SkinList.Text.flags[i]
				if i < Count then
					Flags = Flags .. ","
				end
			end
			Text:SetFont(Font, Size, Flags)
		end
	end
	NS:SetViewport()
end
NS:AddOption("LoadSkin", LoadSkin)

--local Resolution, ViewportOverlay
function NS:SetViewport(fullscreen)
	if InCombatLockdown() then return end
	local bottom = fullscreen and 0 or NS:GetOption("Metrics").RampartTop * 0.63
	WorldFrame:ClearAllPoints()
	WorldFrame:SetPoint("TOPLEFT", 0, 0)
	WorldFrame:SetPoint("BOTTOMRIGHT", 0, bottom)
end

----------------------------------------------------------------------------------------------------------
--						Mini Buttons						--
----------------------------------------------------------------------------------------------------------
local Init = true
local function ManageMiniButtons() -- ALL button settings are in SavedVariables
	local Settings = NS:GetSettings("UI").buttontrays
	local Control = NS:GetFrame("Utility", "ButtonTray1")
	for i=1, #Settings do
		if Init then
			Init = nil
			C_Timer.After(2, function() NS:GetOption("PopulateTray")(i, Settings[i]) end)
		else
			NS:GetOption("PopulateTray")(i, Settings[i])
		end
	end
--	NS:GetOption("SelectTray")(1)
end
NS:AddOption("ManageMiniButtons", ManageMiniButtons)

local function InitialiseFramesCharacter(self)
	ConfirmDelete()
	LoadSkin()
	CombatColorsInit()
	ManageMiniButtons()
	NS:GetFrame("Utility", "MiniButton"):SetPortrait()

	for k,v in pairs(NS:GetFrame("Base")) do
		tinsert(ColourFrames, v)
	end
	tinsert(ColourFrames, NS:GetFrame("Context", "Parent"))

--	NS:CreateContextMenu()
--	NS.MapInitialise()
--	if NS.CreateTrinketz then
--		NS:CreateTrinketz()
--	end
--	NS:LoadBuffWatcher()

end

NS:RegisterForInit("PLAYER_LOGIN", "InitialiseFramesCharacter", 2, InitialiseFramesCharacter)
